#!/usr/bin/env python3

# faster_rhb_udp_fanout.py

import argparse

import socket
import struct
import syslog

import sockpipe

FASTER_DATA_SIZE = 8192

DEFAULT_LOOPBACK_IP_ADDRESS = '127.0.0.1'
DEFAULT_UDP_PORT = 6666

DEFAULT_MULTICAST_GROUP = '239.42.42.42'

MCAST_TTL = 1 # do not cross LAN borders
MCAST_FASTER_DATA_PORT = 4242


def server_setup(args):
    # setup listening socket
    unicast_s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    unicast_s.bind((args.addr, args.port))

    # setup multicast sending socket
    multicast_s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    multicast_s.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, MCAST_TTL)
    multicast_addr = (args.group,MCAST_FASTER_DATA_PORT)

    return unicast_s, multicast_s, multicast_addr

    
def client_setup(args):
    # setup listening socket
    multicast_s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    multicast_s.bind(('', MCAST_FASTER_DATA_PORT)) # interface address ?    
    multicast_request = socket.inet_aton(args.group) + socket.inet_aton('0.0.0.0')
    multicast_s.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, bytes(multicast_request))

    # setup sending socket
    unicast_s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
    unicast_addr = (args.addr, args.port)

    return multicast_s, unicast_s, unicast_addr


def main(args):

    if args.server:
        network_setup = server_setup
    else:
        network_setup = client_setup
        
    in_socket, out_socket, out_addr = network_setup(args)

    syslog.syslog('piping udp data to {}:{}'.format(out_addr[0], out_addr[1]))

    try:
        sc = sockpipe.SocketPipe(in_socket, out_socket, out_addr)
        sc.loop()
    except KeyboardInterrupt as ex:
        pass
    finally:
        out_socket.close()
        in_socket.close()
        print('done.')


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Faster/RHB udp fanout over multicast')
    
    parser.add_argument('--server',           default=False,                                  help='act as server', action='store_true')
    parser.add_argument('--group', nargs='?', default=DEFAULT_MULTICAST_GROUP,     type=str,  help='multicast group address')
    parser.add_argument('--addr',  nargs='?', default=DEFAULT_LOOPBACK_IP_ADDRESS, type=str,  help='unicast address')
    parser.add_argument('--port',  nargs='?', default=DEFAULT_UDP_PORT,            type=int,  help='unicast udp port')
    
    args = parser.parse_args()
    
    main(args)


